@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # This file is part of Panucci.
-# Copyright (c) 2008-2009 The Panucci Audiobook and Podcast Player Project
+# Copyright (c) 2008-2011 The Panucci Project
 #
 # Panucci is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,9 +19,10 @@
 # _Heavily_ inspired by gPodder's dbsqlite
 #
 
+from __future__ import absolute_import
+
 import os.path
 import logging
-import string
 import time
 
 log = logging.getLogger('panucci.dbsqlite')
@@ -31,8 +32,7 @@ try:
 except ImportError:
     log.critical('Error importing sqlite, FAIL!')
 
-from settings import settings
-from simplegconf import gconf
+import panucci
 
 class Storage(object):
     def __init__(self, db_file):
@@ -120,7 +120,7 @@ class Storage(object):
         if conditions:
             sql += ' WHERE '
 
-        sql += string.join(conditions, ' AND ')
+        sql += ' AND '.join(conditions)
 
         cursor = self.cursor()
         cursor.execute( sql, args )
@@ -278,13 +278,13 @@ class Storage(object):
 
     def get_playlist_id(self, filepath, create_new=False, update_time=False):
         """ Get a playlist_id by it's filepath
-                create_new: if True it will create a new playlist 
+                create_new: if True it will create a new playlist
                     entry if none exists for the filepath.
                 update_time: if True it updates the timestamp for the
                     playlist entry of the filepath. """
 
         if self.playlist_exists(filepath):
-            playlist_id = self.__get__( 
+            playlist_id = self.__get__(
                 'SELECT playlist_id FROM playlists WHERE filepath = ?',
                 filepath )[0]
         elif create_new:
@@ -376,5 +376,5 @@ class Storage(object):
 
         return row
 
-db = Storage(os.path.expanduser(settings.db_location))
+db = Storage(os.path.expanduser(panucci.DATABASE_FILE))
 
