@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # This file is part of Panucci.
-# Copyright (c) 2008-2009 The Panucci Audiobook and Podcast Player Project
+# Copyright (c) 2008-2011 The Panucci Project
 #
 # Panucci is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,25 +17,36 @@
 # along with Panucci.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import absolute_import
+
 import gobject, gtk
 import time
 import os.path
 import os
 import re
+import dbus
 import logging
 from hashlib import md5
 from xml.sax.saxutils import escape
 
-import util
-from dbsqlite import db
-from settings import settings
-from simplegconf import gconf
-from services import ObservableService
+import panucci
 
-_ = lambda x: x
+from panucci.dbsqlite import db
+from panucci.services import ObservableService
+
+from panucci import util
+
+def is_supported(uri):
+    filename, extension = os.path.splitext(uri)
+    if extension.startswith('.'):
+        extension = extension[1:]
+
+    return (extension.lower() in panucci.EXTENSIONS)
 
 class Playlist(ObservableService):
-    signals = [ 'new_track', 'new_track_metadata', 'file_queued' ]
+    signals = [ 'new-track-loaded', 'new-metadata-available', 'file_queued',
+                'bookmark_added', 'seek-requested', 'end-of-playlist',
+                'playlist-to-be-overwritten', 'stop-requested', 'reset-playlist' ]
 
     def __init__(self):
         self.__log = logging.getLogger('panucci.playlist.Playlist')
@@ -45,7 +56,8 @@ class Playlist(ObservableService):
         self.__queue.register(
             'current_item_changed', self.on_queue_current_item_changed )
 
-        self.reset_playlist()
+        self.filepath = None
+        self._id = None
 
     def reset_playlist(self):
         """ Sets the playlist to a default "known" state """
@@ -53,8 +65,8 @@ class Playlist(ObservableService):
         self.filepath = None
         self._id = None
         self.__queue.clear()
-        self.__bookmarks_model = None
-        self.__bookmarks_model_changed = True
+        self.stop(None, False)
+        self.notify('reset-playlist', caller=self.reset_playlist)
 
     @property
     def id(self):
@@ -86,7 +98,7 @@ class Playlist(ObservableService):
     def print_queue_layout(self):
         """ This helps with debugging ;) """
         for item in self.__queue:
-            print str(item), item.reported_filepath
+            print str(item), item.playlist_reported_filepath
             for bookmark in item.bookmarks:
                 print '\t', str(bookmark), bookmark.bookmark_filepath
 
@@ -107,20 +119,20 @@ class Playlist(ObservableService):
         # copy the bookmarks over to new playlist
         db.remove_all_bookmarks(self.id)
         self.__queue.set_new_playlist_id(self.id)
-        
+
         return True
 
     def save_temp_playlist(self):
-        filepath = os.path.expanduser(settings.temp_playlist)
-        return self.save_to_new_playlist(filepath)
+        return self.save_to_new_playlist(panucci.PLAYLIST_FILE)
 
     def on_queue_current_item_changed(self):
-        self.send_new_metadata( self.on_queue_current_item_changed )
-        self.notify( 'new_track', caller=self.on_queue_current_item_changed )
+        self.notify( 'new-track-loaded',
+                     caller=self.on_queue_current_item_changed )
+        self.notify( 'new-metadata-available',
+                     caller=self.on_queue_current_item_changed )
 
-    def send_new_metadata(self, caller=None):
-        self.notify( 'new_track_metadata', self.get_file_metadata(),
-            caller=caller or self.send_new_metadata )
+    def send_metadata(self):
+        self.notify( 'new-metadata-available', caller=self.send_metadata )
 
     def quit(self):
         self.__log.debug('quit() called.')
@@ -133,17 +145,26 @@ class Playlist(ObservableService):
     ######################################
 
     def __load_from_bookmark( self, item_id, bookmark ):
-        self.__queue.current_item_position = self.__queue.index(item_id)
+        new_pos = self.__queue.index(item_id)
+        same_pos = self.__queue.current_item_position == new_pos
+        self.__queue.current_item_position = new_pos
 
         if bookmark is None:
             self.__queue.current_item.seek_to = 0
         else:
             self.__queue.current_item.seek_to = bookmark.seek_position
 
+            # if we don't request a seek nothing will happen
+            if same_pos:
+                self.notify( 'seek-requested', bookmark.seek_position,
+                    caller=self.__load_from_bookmark )
+
         return True
 
     def load_from_bookmark_id( self, item_id=None, bookmark_id=None ):
         item, bookmark = self.__queue.get_bookmark(item_id, bookmark_id)
+        if str(self.__queue.current_item) != item_id:
+            self.notify('stop-requested', caller=self.load_from_bookmark_id)
 
         if item is not None:
             return self.__load_from_bookmark( str(item), bookmark )
@@ -170,17 +191,19 @@ class Playlist(ObservableService):
             return self.load_from_bookmark_id( item_id, bookmark_id )
 
     def save_bookmark( self, bookmark_name, position ):
-        self.__queue.current_item.save_bookmark(
-            bookmark_name, position, resume_pos=False )
-        self.__bookmarks_model_changed = True
+        if self.__queue.current_item is not None:
+            self.__queue.current_item.save_bookmark( bookmark_name, position,
+                                                     resume_pos=False )
+            self.notify( 'bookmark_added', str(self.__queue.current_item),
+                         bookmark_name, position, caller=self.save_bookmark )
 
     def update_bookmark(self, item_id, bookmark_id, name=None, seek_pos=None):
         item, bookmark = self.__queue.get_bookmark(item_id, bookmark_id)
-        
+
         if item is None:
             self.__log.warning('No such item id (%s)', item_id)
             return False
-        
+
         if bookmark_id is not None and bookmark is None:
             self.__log.warning('No such bookmark id (%s)', bookmark_id)
             return False
@@ -189,6 +212,9 @@ class Playlist(ObservableService):
             if name and item.title != name:
                 item.title = name
                 self.__queue.modified = True
+            if self.__queue.current_item == item:
+                self.notify( 'new-metadata-available',
+                             caller=self.update_bookmark )
         else:
             bookmark.timestamp = time.time()
 
@@ -236,40 +262,43 @@ class Playlist(ObservableService):
             return False
         else:
             return self.remove_bookmark( item_id, bookmark_id )
-                    
-
-    def generate_bookmark_model(self, include_resume_marks=False):
-        self.__bookmarks_model = gtk.TreeStore(
-            # uid, name, position
-            gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
-
-        for item in self.__queue:
-            title = util.pretty_filename(
-                item.filepath ) if item.title is None else item.title
-            row = [ str(item), title, None ]
-            parent = self.__bookmarks_model.append( None, row )
-
-            for bkmk in item.bookmarks:
-                if not bkmk.is_resume_position or include_resume_marks:
-                    row = [ str(bkmk), bkmk.bookmark_name,
-                        util.convert_ns(bkmk.seek_position) ]
-                    self.__bookmarks_model.append( parent, row )
-
-    def get_bookmark_model(self, include_resume_marks=False):
-        if self.__bookmarks_model is None or self.__bookmarks_model_changed:
-            self.__log.debug('Generating new bookmarks model')
-            self.generate_bookmark_model(include_resume_marks)
-            self.__bookmarks_model_changed = False
-        else:
-            self.__log.debug('Using cached bookmarks model')
-
-        return self.__bookmarks_model
 
     def move_item( self, from_row, to_row ):
         self.__log.info('Moving item from position %d to %d', from_row, to_row)
         assert isinstance(from_row, int) and isinstance(to_row, int)
         self.__queue.move_item(from_row, to_row)
 
+    #####################################
+    # Model-builder functions
+    #####################################
+
+    def get_item_by_id(self, item_id):
+        """ Gets a PlaylistItem from it's unique id """
+
+        item, bookmark = self.__queue.get_bookmark(item_id, None)
+        if item is None:
+            self.__log.warning('Cannot get item for id: %s', item_id)
+
+        return item
+
+    def get_playlist_item_ids(self):
+        """ Returns an iterator which yields a tuple which contains the
+            item's unique ID and a dict of interesting data (currently
+            just the title). """
+        
+        for item in self.__queue:
+            yield str(item), { 'title' : item.title }
+
+    def get_bookmarks_from_item_id(self, item_id, include_resume_marks=False):
+        """ Returns an iterator which yields the following data regarding a
+            bookmark: ( bookmark id, a custom name, the seek position ) """
+
+        item = self.get_item_by_id( item_id )
+        if item is not None:
+            for bkmk in item.bookmarks:
+                if not bkmk.is_resume_position or include_resume_marks:
+                    yield str(bkmk), bkmk.bookmark_name, bkmk.seek_position
+
     ######################################
     # File-related convenience functions
     ######################################
@@ -316,37 +345,39 @@ class Playlist(ObservableService):
         """ Detects filepath's filetype then loads it using
             the appropriate loader function """
         self.__log.debug('Attempting to load %s', filepath)
-
-        # Again, a very KLUDGY workaround (thp, 2010-03-11)
-        if filepath.startswith('/http:/'):
-            filepath = 'http://' + filepath[7:]
-
+        _play = self.__queue.is_empty()
         error = False
-        self.reset_playlist()
-        self.filepath = filepath
-        self.__queue.playlist_id = self.id
 
-        parsers = { 'm3u': M3U_Playlist, 'pls': PLS_Playlist }
-        extension = util.detect_filetype(filepath)
-        if parsers.has_key(extension): # importing a playlist
-            self.__log.info('Loading playlist file (%s)', extension)
-            parser = parsers[extension](self.filepath, self.__queue)
+        if os.path.isdir(filepath):
+            self.load_directory(filepath, True)
+        else:
+            parsers = { 'm3u': M3U_Playlist, 'pls': PLS_Playlist }
+            extension = util.detect_filetype(filepath)
+            if parsers.has_key(extension): # importing a playlist
+                self.__log.info('Loading playlist file (%s)', extension)
+                parser = parsers[extension](filepath, self.__queue)
 
-            if parser.parse(filepath):
-                self.__queue = parser.get_queue()
-            else:
-                return False
-        else:                          # importing a single file
-            error = not self.append(filepath, dont_notify=True)
+                if parser.parse(filepath):
+                    self.__queue = parser.get_queue()
+                    self.__file_queued( filepath, True, False )
+                else:
+                   return False
+            else:                          # importing a single file
+                error = not self.append(filepath, notify=False)
 
-        self.__queue.disable_notifications = True
-        self.load_from_resume_bookmark()
-        self.__queue.disable_notifications = False
-
-        self.__queue.modified = os.path.expanduser(
-            settings.temp_playlist ) == self.filepath
-
-        self.send_new_metadata( caller=self.load )
+        # if we let the queue emit a current_item_changed signal (which will
+        # happen if load_from_bookmark changes the current track), the player
+        # will start playing and ingore the resume point
+        if _play:
+            self.filepath = filepath
+            self.__queue.playlist_id = self.id
+            self.__queue.disable_notifications = True
+            self.load_from_resume_bookmark()
+            self.__queue.disable_notifications = False
+            self.__queue.modified = True
+            self.notify( 'stop-requested', caller=self.load )
+            self.notify( 'new-track-loaded', caller=self.load )
+            self.notify( 'new-metadata-available', caller=self.load )
 
         return not error
 
@@ -354,59 +385,71 @@ class Playlist(ObservableService):
         recent = self.get_recent_files(max_files=1)
         if recent:
             self.load(recent[0])
-
         return bool(recent)
 
-    def __file_queued(self, filepath, successfull):
+    def __file_queued(self, filepath, successfull, notify):
         if successfull:
-            self.__bookmarks_model_changed = True
-            self.notify(
-              'file_queued', filepath, successfull, caller=self.__file_queued )
+            self.notify( 'file_queued', filepath, successfull, notify,
+                caller=self.__file_queued )
 
         return successfull
 
-    def append(self, filepath, dont_notify=False):
+    def append(self, filepath, notify=True):
         self.__log.debug('Attempting to queue file: %s', filepath)
         success = self.__queue.append(
             PlaylistItem.create_by_filepath(filepath, filepath) )
-
-        return success if dont_notify else self.__file_queued(filepath,success)
+        return self.__file_queued( filepath, success, notify)
 
     def insert(self, position, filepath ):
         self.__log.debug(
             'Attempting to insert %s at position %s', filepath, position )
-        return self.__file_queued( filepath, self.__queue.insert(
-            position, PlaylistItem.create_by_filepath(filepath, filepath) ))
+        return self.__file_queued( filepath, self.__queue.insert( position,
+            PlaylistItem.create_by_filepath(filepath, filepath)), True )
 
     def load_directory(self, directory, append=False):
         self.__log.debug('Attempting to load directory "%s"', directory)
 
-        if not append:
-            self.reset_playlist()
-
-        if os.path.isdir(directory):
-            self.filepath = settings.temp_playlist
-            self.__queue.playlist_id = self.id
-            items = []
-
-            for item in os.listdir(directory):
-                filepath = os.path.join( directory, item )
-                if os.path.isfile(filepath) and util.is_supported(filepath):
-                    items.append(filepath)
-
-            items.sort()
-            for item in items:
-                self.append( item, dont_notify=True )
-        else:
+        if not os.path.isdir(directory):
             self.__log.warning('"%s" is not a directory.', directory)
             return False
 
+        if not append:
+            if self.notify( 'playlist-to-be-overwritten',
+                            caller=self.load_directory ):
+                self.reset_playlist()
+            else:
+                self.__log.info('Directory load aborted by user.')
+                return False
+
+        self.filepath = panucci.PLAYLIST_FILE
+        self.__queue.playlist_id = self.id
+
+        items = []
+        potential_items = os.listdir(directory)
+        potential_items.sort()
+
+        for item in potential_items:
+            filepath = os.path.join( directory, item )
+            if os.path.isfile(filepath) and is_supported(filepath):
+                items.append(filepath)
+
+        items.sort()
+        for item in items:
+            self.append( item, notify=False )
+
+        if not append:
+            self.on_queue_current_item_changed()
+
         return True
 
     ##################################
     # Playlist controls
     ##################################
 
+    def set_seek_to(self, position):
+        """Set the seek-to position for the current track"""
+        self.__queue.current_item.seek_to = (10**9) * position
+
     def play(self):
         """ This gets called by the player to get
                 the last time the file was paused """
@@ -421,19 +464,18 @@ class Playlist(ObservableService):
     def stop(self, position, save_resume_point=True):
         """ This should be run when the program is closed
                 or if the user switches playlists """
-    
         self.remove_resume_bookmarks()
         if not self.is_empty and save_resume_point:
             self.__queue.current_item.save_bookmark(
                 _('Auto Bookmark'), position, True )
 
-    def skip(self, skip_by=None, skip_to=None, dont_loop=False):
+    def skip(self, skip_by=None, skip_to=None, loop=True):
         """ Skip to another track in the playlist.
             Use either skip_by or skip_to, skip_by has precedence.
                 skip_to: skip to a known playlist position
                 skip_by: skip by n number of episodes (positive or negative)
-                dont_loop: applies only to skip_by, if we're skipping past
-                    the last track loop back to the begining.
+                loop: loop if the track requested lays out of
+                      the 0 to queue_length-1 boundary.
         """
         if not self.__queue:
             return False
@@ -441,35 +483,51 @@ class Playlist(ObservableService):
         current_item = self.__queue.current_item_position
 
         if skip_by is not None:
-            if dont_loop:
-                skip = current_item + skip_by
-            else:
-                skip = ( current_item + skip_by ) % self.queue_length
+            skip = current_item + skip_by
         elif skip_to is not None:
             skip = skip_to
         else:
+            skip = 0
             self.__log.warning('No skip method provided...')
 
-        if not ( 0 <= skip < self.queue_length ):
-            self.__log.warning(
-                'Can\'t skip to non-existant file. (requested=%d, total=%d)',
-                skip, self.queue_length )
-            return False
+        if not 0 <= skip < self.queue_length:
+            self.notify( 'end-of-playlist', loop, caller=self.skip )
 
+            if not loop:
+                self.__log.warning( "Can't skip to non-existant file w/o loop."
+                                    " (requested=%d, total=%d)", skip,
+                                    self.queue_length )
+                return False
+            else:
+                # If skip_by is given as an argument, we assume the user knows
+                # what they're doing. Ie. if the queue length is 5, current
+                # track is 3 and they pass skip_by=-9, then they'll end up
+                # at 4. On the other hand, if skip_to is passed then we skip
+                # back to 0 because in that case the user must enter a number
+                # from 0 to queue_length-1, anything else is an error.
+                if skip_by is not None:
+                    skip %= self.queue_length
+                else:
+                    skip = 0
+
+        self.notify('stop-requested', caller=self.skip)
         self.__queue.current_item_position = skip
-        self.__log.debug('Skipping to file %d (%s)', skip,
-            self.__queue.current_item.filepath )
+        self.__log.debug( 'Skipping to file %d (%s)', skip,
+                          self.__queue.current_item.filepath )
 
         return True
 
+    def get_current_item(self):
+        return self.__queue.current_item
+
     def next(self):
         """ Move the playlist to the next track.
             False indicates end of playlist. """
-        return self.skip( skip_by=1, dont_loop=True )
+        return self.skip( skip_by=1, loop=False )
 
     def prev(self):
         """ Same as next() except moves to the previous track. """
-        return self.skip( skip_by=-1, dont_loop=True )
+        return self.skip( skip_by=-1, loop=False )
 
 
 class Queue(list, ObservableService):
@@ -485,6 +543,9 @@ class Queue(list, ObservableService):
         self.modified = False # Has the queue been modified?
         self.disable_notifications = False
         self.__current_item_position = 0
+        # This is a hack and WILL BE REPLACED WITH SOMETHING BETTER.
+        # it's here to speed up the get_item function
+        self.__mapping_dict = {}
         list.__init__(self)
 
     def __get_current_item_position(self):
@@ -492,7 +553,7 @@ class Queue(list, ObservableService):
 
     def __set__current_item_position(self, new_value):
 
-        # set the new position before notify()'ing 
+        # set the new position before notify()'ing
         # or else we'll end up load the old file's metadata
         old_value = self.__current_item_position
         self.__current_item_position = new_value
@@ -521,7 +582,8 @@ class Queue(list, ObservableService):
         assert isinstance( item, PlaylistItem )
         item.playlist_id = self.playlist_id
 
-        if os.path.isfile(item.filepath) and util.is_supported(item.filepath) or item.filepath.startswith('http://'):
+        if '://' in item.filepath or (os.path.isfile(item.filepath) and \
+                is_supported(item.filepath)):
             self.modified = True
             return True
         else:
@@ -555,15 +617,27 @@ class Queue(list, ObservableService):
     def clear(self):
         """ Reset the the queue to a known state """
 
+        try:
+            items = self.__mapping_dict.values()
+            for item in items:
+              list.remove(self, item)
+        except:
+          pass
         self[:] = []
         self.playlist_id = None
-        self.modified = False
+        self.modified = True
         self.__current_item_position = 0
+        self.__mapping_dict = {}
 
     def get_item(self, item_id):
-        if self.count(item_id):
-            return self[self.index(item_id)]
+        return self.__mapping_dict.get(item_id)
 
+    def is_empty(self):
+        if self.__mapping_dict:
+            return False
+        else:
+            return True
+  
     def get_bookmark(self, item_id, bookmark_id):
         item = self.get_item(item_id)
 
@@ -602,6 +676,9 @@ class Queue(list, ObservableService):
                 if i.filepath == item.filepath:
                     i.is_modified = True
                     i.duplicate_id += 1
+
+            # to be safe rebuild self.__mapping_dict
+            self.__mapping_dict = dict([(str(i),i) for i in self])
         elif not self.__count_dupe_items(self[:position], item):
             # there are no other items like this one so it's *safe* to load
             # bookmarks without a potential conflict, but there's a good chance
@@ -612,6 +689,7 @@ class Queue(list, ObservableService):
         if position <= self.current_item_position:
             self.__current_item_position += 1
 
+        self.__mapping_dict[str(item)] = item
         list.insert(self, position, item)
         return True
 
@@ -622,6 +700,7 @@ class Queue(list, ObservableService):
         item.duplicate_id = self.__count_dupe_items(self, item)
         item.load_bookmarks()
 
+        self.__mapping_dict[str(item)] = item
         list.append(self, item)
         return True
 
@@ -632,6 +711,7 @@ class Queue(list, ObservableService):
             if self.index(item) < self.current_item_position:
                 self.__current_item_position -= 1
 
+            del self.__mapping_dict[str(item)]
             list.remove(self, item)
 
     def extend(self, items):
@@ -647,31 +727,47 @@ class PlaylistItem(object):
     def __init__(self):
         self.__log = logging.getLogger('panucci.playlist.PlaylistItem')
 
-        # metadata that's pulled from the playlist file (pls/extm3u)
-        self.reported_filepath = None
-        self.title = None
-        self.length = None
-
+        self.__filepath = None
+        self.__metadata = None
         self.playlist_id = None
-        self.filepath = None
         self.duplicate_id = 0
         self.seek_to = 0
 
+        # metadata that's pulled from the playlist file (pls/extm3u)
+        self.playlist_reported_filepath = None
+        self.playlist_title = None
+        self.playlist_length = None
+
         # a flag to determine whether the item's bookmarks need updating
         # ( used for example, if the duplicate_id is changed )
         self.is_modified = False
         self.bookmarks = []
 
+    def __set_filepath(self, fp):
+        if fp != self.__filepath:
+            self.__filepath = fp
+            self.__metadata = FileMetadata(self.filepath)
+            # Don't extract Metadata right away, this makes opening large
+            #   playlists _very_ slow. TODO: do this intelligently AND
+            #   perhaps del the Metadata object if the file is no longer
+            #   being used.
+            #self.__metadata.extract_metadata()
+
+    def __get_filepath(self):
+        return self.__filepath
+
+    filepath = property( __get_filepath, __set_filepath )
+
     @staticmethod
     def create_by_filepath(reported_filepath, filepath):
         item = PlaylistItem()
-        item.reported_filepath = reported_filepath
+        item.playlist_reported_filepath = reported_filepath
         item.filepath = filepath
         return item
 
     def __eq__(self, b):
         if isinstance( b, PlaylistItem ):
-            return ( self.filepath == b.filepath and 
+            return ( self.filepath == b.filepath and
                      self.duplicate_id == b.duplicate_id )
         elif isinstance( b, str ):
             return str(self) == b
@@ -685,19 +781,48 @@ class PlaylistItem(object):
 
     @property
     def metadata(self):
-        """ Metadata is only needed once, so fetch it on-the-fly
-            If needed this could easily be cached at the cost of wasting a 
-            bunch of memory """
+        """ Returns a dict of metadata, wooo. """
 
-        m = FileMetadata(self.filepath)
-        metadata = m.get_metadata()
-        del m   # *hopefully* save some memory
+        metadata = self.__metadata.get_metadata()
+        metadata['title'] = self.title
         return metadata
 
     @property
     def filetype(self):
         return util.detect_filetype(self.filepath)
 
+    def __get_title(self):
+        """ Get the title of item, priority is (most important first):
+            1. the title given in playlist metadata
+            2. the title in the file's metadata (ex. ID3)
+            3. a "pretty" version of the filename """
+
+        if self.playlist_title is not None:
+            return self.playlist_title
+        elif self.__metadata.title:
+            return self.__metadata.title
+        else:
+            return util.pretty_filename(self.filepath)
+
+    # For now set the "playlist_title" because it has highest priority in the
+    # __get_title method. We might evenually want to create a separate
+    # __custom_title to preserve the playlist_title.
+    title = property(__get_title, lambda s,v: setattr(s, 'playlist_title', v))
+
+    @property
+    def length(self):
+        """ Get the lenght of the item priority is (most important first):
+            1. length as reported by mutagen
+            2. length found in playlist metadata
+            3. otherwise -1 when unknown """
+
+        if self.__metadata.length:
+            return self.__metadata.length
+        elif self.playlist_length:
+            return self.playlist_length
+        else:
+            return -1
+
     def load_bookmarks(self):
         self.bookmarks = db.load_bookmarks(
             factory                 = Bookmark().load_from_dict,
@@ -721,8 +846,9 @@ class PlaylistItem(object):
     def delete_bookmark(self, bookmark_id):
         """ WARNING: if bookmark_id is None, ALL bookmarks will be deleted """
         if bookmark_id is None:
-            self.__log.debug(
-                'Deleting all bookmarks for %s', self.reported_filepath )
+            self.__log.debug( 'Deleting all bookmarks for %s',
+                              self.playlist_reported_filepath )
+
             for bkmk in self.bookmarks:
                 bkmk.delete()
         else:
@@ -778,6 +904,8 @@ class Bookmark(object):
     def __eq__(self, b):
         if isinstance(b, str):
             return str(self) == b
+        elif b is None:
+            return False
         else:
             self.__log.warning('Unsupported comparison: %s', type(b))
             return False
@@ -804,7 +932,8 @@ class FileMetadata(object):
     """ A class to hold all information about the file that's currently being
         played. Basically it takes care of metadata extraction... """
 
-    coverart_names = ['cover', 'cover.jpg', 'cover.png']
+    coverart_names = ['cover', 'front', 'albumart', 'back']
+    coverart_extensions = ['.png', '.jpg', '.jpeg']
     tag_mappings = {
         'mp4': { '\xa9nam': 'title',
                  '\xa9ART': 'artist',
@@ -833,7 +962,35 @@ class FileMetadata(object):
 
         self.__metadata_extracted = False
 
+    def _ask_gpodder_for_metadata(self):
+        GPO_NAME = 'org.gpodder'
+        GPO_PATH = '/podcasts'
+        GPO_INTF = 'org.gpodder.podcasts'
+
+        bus = dbus.SessionBus()
+        try:
+            if bus.name_has_owner(GPO_NAME):
+                o = bus.get_object(GPO_NAME, GPO_PATH)
+                i = dbus.Interface(o, GPO_INTF)
+                episode, podcast = i.get_episode_title(self.__filepath)
+                if episode and podcast:
+                    self.title = episode
+                    self.artist = podcast
+                    self.__metadata_extracted = True
+                    return True
+        except Exception, e:
+            self.__log.debug('Cannot get metadata from gPodder: %s', str(e))
+
+        return False
+
     def extract_metadata(self):
+        self.__log.debug('Extracting metadata for %s', self.__filepath)
+
+        if self._ask_gpodder_for_metadata():
+            if self.coverart is None:
+                self.coverart = self.__find_coverart()
+            return
+
         filetype = util.detect_filetype(self.__filepath)
 
         if filetype == 'mp3':
@@ -851,9 +1008,11 @@ class FileMetadata(object):
 
         try:
             metadata = meta_parser.Open(self.__filepath)
+            self.__metadata_extracted = True
         except Exception, e:
             self.title = util.pretty_filename(self.__filepath)
             self.__log.exception('Error running metadata parser...')
+            self.__metadata_extracted = False
             return False
 
         self.length = metadata.info.length * 10**9
@@ -873,7 +1032,7 @@ class FileMetadata(object):
 
                 if self.tag_mappings[filetype][tag] != 'coverart':
                     try:
-                        value = escape(str(value))
+                        value = escape(str(value).strip())
                     except Exception, e:
                         self.__log.exception(
                           'Could not convert tag (%s) to escaped string', tag )
@@ -885,36 +1044,89 @@ class FileMetadata(object):
 
                 setattr( self, self.tag_mappings[filetype][tag], value )
 
-        if not str(self.title).strip():
-            self.title = util.pretty_filename(self.__filepath)
-
         if self.coverart is None:
             self.coverart = self.__find_coverart()
 
     def __find_coverart(self):
         """ Find coverart in the same directory as the filepath """
+
+        if '://' in self.__filepath and \
+                not self.__filepath.startswith('file://'):
+            # No cover art for streaming at the moment
+            return None
+
         directory = os.path.dirname(self.__filepath)
-        for cover in self.coverart_names:
-            c = os.path.join( directory, cover )
-            if os.path.isfile(c):
-                try:
-                    f.open(c,'r')
-                    binary_coverart = f.read()
-                    f.close()
-                    return binary_coverart
-                except:
-                    pass
+        for cover in self.__find_coverart_filepath(directory):
+            self.__log.debug('Trying to load coverart from %s', cover)
+            try:
+                f = open(cover,'r')
+            except:
+                self.__log.exception('Could not open coverart file %s', cover )
+                continue
+
+            binary_coverart = f.read()
+            f.close()
+
+            if self.__test_coverart(binary_coverart):
+                return binary_coverart
+
         return None
 
+    def __test_coverart(self, data):
+        """ tests to see if the file is a proper image file that can be loaded
+            into a gtk.gdk.Pixbuf """
+
+        import gtk.gdk
+        l = gtk.gdk.PixbufLoader()
+        try:
+            l.write(data)
+            l.close()
+            rtn = True
+        except:
+            rtn = False
+
+        return rtn
+
+    def __find_coverart_filepath(self, directory):
+        """ finds the path of potential coverart files """
+        dir_filelist = []
+        possible_matches = []
+
+        # build the filelist
+        for f in os.listdir(directory):
+            if os.path.isfile(os.path.join(directory,f)):
+                dir_filelist.append(os.path.splitext(f))
+
+        # first pass, check for known filenames
+        for f,ext in dir_filelist[:]:
+            if f.lower() in self.coverart_names and \
+                ext.lower() in self.coverart_extensions:
+                possible_matches.append((f,ext))
+                dir_filelist.remove((f,ext))
+
+        # second pass, check for known filenames without extensions
+        for f,ext in dir_filelist[:]:
+            if f.lower() in self.coverart_names and ext == '':
+                possible_matches.append((f,ext))
+                dir_filelist.remove((f,ext))
+
+        # third pass, check for any image file
+        for f,ext in dir_filelist[:]:
+            if ext.lower() in self.coverart_extensions:
+                possible_matches.append((f,ext))
+                dir_filelist.remove((f,ext))
+
+        # yield the results
+        for f,ext in possible_matches:
+            yield os.path.join( directory, f+ext )
+
     def get_metadata(self):
         """ Returns a dict of metadata """
 
         if not self.__metadata_extracted:
-            self.__log.debug('Extracting metadata for %s', self.__filepath)
             self.extract_metadata()
-            self.__metadata_extracted = True
 
-        metadata = { 
+        metadata = {
             'title':    self.title,
             'artist':   self.artist,
             'album':    self.album,
@@ -947,7 +1159,7 @@ class PlaylistFile(object):
 
             self.__log.exception( 'Error opening file: %s', filepath)
             return False
-    
+
         return True
 
     def __close_file(self):
@@ -1025,7 +1237,7 @@ class PlaylistFile(object):
         pass
 
     def _add_playlist_item(self, item):
-        path = self.get_absolute_filepath(item.reported_filepath)
+        path = self.get_absolute_filepath(item.playlist_reported_filepath)
         if path is not None and os.path.isfile(path):
             item.filepath = path
             self._items.append(item)
@@ -1047,23 +1259,24 @@ class M3U_Playlist(PlaylistFile):
             if match is not None:
                 length, title = match.groups()
                 try: length = int(length)
-                except: pass
-                self.current_item.length = length
-                self.current_item.title = title
+                except: length = -1
+                self.current_item.playlist_length = length
+                self.current_item.playlist_title = title
         elif line.startswith('#'):
             pass # skip comments
         elif line:
             path = self.get_absolute_filepath( line )
             if path is not None:
                 if os.path.isfile( path ):
-                    self.current_item.reported_filepath = line
+                    self.current_item.playlist_reported_filepath = line
                     self._add_playlist_item(self.current_item)
                     self.current_item = PlaylistItem()
                 elif os.path.isdir( path ):
                     files = os.listdir( path )
+                    files.sort()
                     for file in files:
                         item = PlaylistItem()
-                        item.reported_filepath = os.path.join(line, file)
+                        item.playlist_reported_filepath=os.path.join(line,file)
                         self._add_playlist_item(item)
 
     def export_hook(self, playlist_items):
@@ -1072,10 +1285,10 @@ class M3U_Playlist(PlaylistFile):
         for item in playlist_items:
             string = ''
             if not ( item.length is None and item.title is None ):
-                length = -1 if item.length is None else item.length
+                length = -1 if item.length is None else int(item.length)
                 title = '' if item.title is None else item.title
                 string += '#EXTINF:%d,%s\n' % ( length, title )
-                
+
             string += '%s\n' % item.filepath
             self._file.write(string)
 
@@ -1112,14 +1325,15 @@ class PLS_Playlist(PlaylistFile):
         elif not self.in_playlist_section:
             pass # don't do anything if we're not in [playlist]
         elif line.lower().startswith('file'):
-            self.current_item.reported_filepath = re.search(
+            self.current_item.playlist_reported_filepath = re.search(
                 item_regex, line).group(2)
         elif line.lower().startswith('title'):
-            self.current_item.title = re.search(item_regex, line).group(2)
+            self.current_item.playlist_title = re.search(
+                                                    item_regex, line).group(2)
         elif line.lower().startswith('length'):
             try: length = int(re.search(item_regex, line).group(2))
-            except: pass
-            self.current_item.length = length
+            except: length = -1
+            self.current_item.playlist_length = length
 
     def parse_eof_hook(self):
         self.__add_current_item()
@@ -1136,5 +1350,3 @@ class PLS_Playlist(PlaylistFile):
             self._file.write('Length%d=%s\n\n' % (i+1, length))
 
         self._file.write('Version=2\n')
-
-
